@@ -66,7 +66,6 @@
 
 
 #include "defs.h"
-#include <string.h>
 #include <ctype.h>
 #include "expression.h"
 #include "value.h"
@@ -148,7 +147,7 @@ static int yylex (void);
 void yyerror (char *);
 
 
-#line 152 "go-exp.c" /* yacc.c:339  */
+#line 151 "go-exp.c" /* yacc.c:339  */
 
 # ifndef YY_NULLPTR
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -273,7 +272,7 @@ extern int yydebug;
 typedef union YYSTYPE YYSTYPE;
 union YYSTYPE
 {
-#line 143 "go-exp.y" /* yacc.c:355  */
+#line 142 "go-exp.y" /* yacc.c:355  */
 
     LONGEST lval;
     struct {
@@ -295,7 +294,7 @@ union YYSTYPE
     struct stoken_vector svec;
   
 
-#line 299 "go-exp.c" /* yacc.c:355  */
+#line 298 "go-exp.c" /* yacc.c:355  */
 };
 # define YYSTYPE_IS_TRIVIAL 1
 # define YYSTYPE_IS_DECLARED 1
@@ -309,7 +308,7 @@ int yyparse (void);
 
 
 /* Copy the second part of user declarations.  */
-#line 164 "go-exp.y" /* yacc.c:358  */
+#line 163 "go-exp.y" /* yacc.c:358  */
 
 /* YYSTYPE gets defined by %union.  */
 static int parse_number (struct parser_state *,
@@ -317,7 +316,7 @@ static int parse_number (struct parser_state *,
 static int parse_go_float (struct gdbarch *gdbarch, const char *p, int len,
 			   DOUBLEST *d, struct type **t);
 
-#line 321 "go-exp.c" /* yacc.c:358  */
+#line 320 "go-exp.c" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -617,13 +616,13 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   243,   243,   244,   247,   254,   255,   260,   264,   268,
-     272,   276,   280,   284,   288,   294,   300,   307,   317,   324,
-     321,   332,   336,   339,   343,   347,   351,   357,   363,   369,
-     373,   377,   381,   385,   389,   393,   397,   401,   405,   409,
-     413,   417,   421,   425,   429,   433,   437,   441,   445,   449,
-     453,   459,   466,   475,   488,   495,   498,   504,   519,   526,
-     543,   561,   573,   579,   585,   601,   656,   658,   665,   678
+       0,   242,   242,   243,   246,   253,   254,   259,   263,   267,
+     271,   275,   279,   283,   287,   293,   299,   306,   316,   323,
+     320,   331,   335,   338,   342,   346,   350,   356,   362,   368,
+     372,   376,   380,   384,   388,   392,   396,   400,   404,   408,
+     412,   416,   420,   424,   428,   432,   436,   440,   444,   448,
+     452,   458,   465,   474,   487,   494,   497,   503,   518,   525,
+     542,   560,   572,   578,   584,   600,   655,   657,   664,   677
 };
 #endif
 
@@ -1546,86 +1545,86 @@ yyreduce:
   switch (yyn)
     {
         case 4:
-#line 248 "go-exp.y" /* yacc.c:1646  */
+#line 247 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, OP_TYPE);
 			  write_exp_elt_type (pstate, (yyvsp[0].tval));
 			  write_exp_elt_opcode (pstate, OP_TYPE); }
-#line 1555 "go-exp.c" /* yacc.c:1646  */
+#line 1554 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 6:
-#line 256 "go-exp.y" /* yacc.c:1646  */
+#line 255 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_COMMA); }
-#line 1561 "go-exp.c" /* yacc.c:1646  */
+#line 1560 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 7:
-#line 261 "go-exp.y" /* yacc.c:1646  */
+#line 260 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_IND); }
-#line 1567 "go-exp.c" /* yacc.c:1646  */
+#line 1566 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 8:
-#line 265 "go-exp.y" /* yacc.c:1646  */
+#line 264 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_ADDR); }
-#line 1573 "go-exp.c" /* yacc.c:1646  */
+#line 1572 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 9:
-#line 269 "go-exp.y" /* yacc.c:1646  */
+#line 268 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_NEG); }
-#line 1579 "go-exp.c" /* yacc.c:1646  */
+#line 1578 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 10:
-#line 273 "go-exp.y" /* yacc.c:1646  */
+#line 272 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_PLUS); }
-#line 1585 "go-exp.c" /* yacc.c:1646  */
+#line 1584 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 11:
-#line 277 "go-exp.y" /* yacc.c:1646  */
+#line 276 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_LOGICAL_NOT); }
-#line 1591 "go-exp.c" /* yacc.c:1646  */
+#line 1590 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 12:
-#line 281 "go-exp.y" /* yacc.c:1646  */
+#line 280 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_COMPLEMENT); }
-#line 1597 "go-exp.c" /* yacc.c:1646  */
+#line 1596 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 13:
-#line 285 "go-exp.y" /* yacc.c:1646  */
+#line 284 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_POSTINCREMENT); }
-#line 1603 "go-exp.c" /* yacc.c:1646  */
+#line 1602 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 14:
-#line 289 "go-exp.y" /* yacc.c:1646  */
+#line 288 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_POSTDECREMENT); }
-#line 1609 "go-exp.c" /* yacc.c:1646  */
+#line 1608 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 15:
-#line 295 "go-exp.y" /* yacc.c:1646  */
+#line 294 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
 			  write_exp_string (pstate, (yyvsp[0].ssym).stoken);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
-#line 1617 "go-exp.c" /* yacc.c:1646  */
+#line 1616 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 16:
-#line 301 "go-exp.y" /* yacc.c:1646  */
+#line 300 "go-exp.y" /* yacc.c:1646  */
     { mark_struct_expression (pstate);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
 			  write_exp_string (pstate, (yyvsp[-1].ssym).stoken);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
-#line 1626 "go-exp.c" /* yacc.c:1646  */
+#line 1625 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 17:
-#line 308 "go-exp.y" /* yacc.c:1646  */
+#line 307 "go-exp.y" /* yacc.c:1646  */
     { struct stoken s;
 			  mark_struct_expression (pstate);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
@@ -1633,232 +1632,232 @@ yyreduce:
 			  s.length = 0;
 			  write_exp_string (pstate, s);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
-#line 1638 "go-exp.c" /* yacc.c:1646  */
+#line 1637 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 18:
-#line 318 "go-exp.y" /* yacc.c:1646  */
+#line 317 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_SUBSCRIPT); }
-#line 1644 "go-exp.c" /* yacc.c:1646  */
+#line 1643 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 19:
-#line 324 "go-exp.y" /* yacc.c:1646  */
+#line 323 "go-exp.y" /* yacc.c:1646  */
     { start_arglist (); }
-#line 1650 "go-exp.c" /* yacc.c:1646  */
+#line 1649 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 20:
-#line 326 "go-exp.y" /* yacc.c:1646  */
+#line 325 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, OP_FUNCALL);
 			  write_exp_elt_longcst (pstate,
 						 (LONGEST) end_arglist ());
 			  write_exp_elt_opcode (pstate, OP_FUNCALL); }
-#line 1659 "go-exp.c" /* yacc.c:1646  */
+#line 1658 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 21:
-#line 333 "go-exp.y" /* yacc.c:1646  */
+#line 332 "go-exp.y" /* yacc.c:1646  */
     { start_arglist (); }
-#line 1665 "go-exp.c" /* yacc.c:1646  */
+#line 1664 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 23:
-#line 340 "go-exp.y" /* yacc.c:1646  */
+#line 339 "go-exp.y" /* yacc.c:1646  */
     { arglist_len = 1; }
-#line 1671 "go-exp.c" /* yacc.c:1646  */
+#line 1670 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 24:
-#line 344 "go-exp.y" /* yacc.c:1646  */
+#line 343 "go-exp.y" /* yacc.c:1646  */
     { arglist_len++; }
-#line 1677 "go-exp.c" /* yacc.c:1646  */
+#line 1676 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 25:
-#line 348 "go-exp.y" /* yacc.c:1646  */
+#line 347 "go-exp.y" /* yacc.c:1646  */
     { (yyval.lval) = end_arglist () - 1; }
-#line 1683 "go-exp.c" /* yacc.c:1646  */
+#line 1682 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 26:
-#line 352 "go-exp.y" /* yacc.c:1646  */
+#line 351 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_MEMVAL);
 			  write_exp_elt_type (pstate, (yyvsp[-2].tval));
 			  write_exp_elt_opcode (pstate, UNOP_MEMVAL); }
-#line 1691 "go-exp.c" /* yacc.c:1646  */
+#line 1690 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 27:
-#line 358 "go-exp.y" /* yacc.c:1646  */
+#line 357 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, UNOP_CAST);
 			  write_exp_elt_type (pstate, (yyvsp[-3].tval));
 			  write_exp_elt_opcode (pstate, UNOP_CAST); }
-#line 1699 "go-exp.c" /* yacc.c:1646  */
+#line 1698 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 28:
-#line 364 "go-exp.y" /* yacc.c:1646  */
+#line 363 "go-exp.y" /* yacc.c:1646  */
     { }
-#line 1705 "go-exp.c" /* yacc.c:1646  */
+#line 1704 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 29:
-#line 370 "go-exp.y" /* yacc.c:1646  */
+#line 369 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_REPEAT); }
-#line 1711 "go-exp.c" /* yacc.c:1646  */
+#line 1710 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 30:
-#line 374 "go-exp.y" /* yacc.c:1646  */
+#line 373 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_MUL); }
-#line 1717 "go-exp.c" /* yacc.c:1646  */
+#line 1716 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 31:
-#line 378 "go-exp.y" /* yacc.c:1646  */
+#line 377 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_DIV); }
-#line 1723 "go-exp.c" /* yacc.c:1646  */
+#line 1722 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 32:
-#line 382 "go-exp.y" /* yacc.c:1646  */
+#line 381 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_REM); }
-#line 1729 "go-exp.c" /* yacc.c:1646  */
+#line 1728 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 33:
-#line 386 "go-exp.y" /* yacc.c:1646  */
+#line 385 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_ADD); }
-#line 1735 "go-exp.c" /* yacc.c:1646  */
+#line 1734 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 34:
-#line 390 "go-exp.y" /* yacc.c:1646  */
+#line 389 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_SUB); }
-#line 1741 "go-exp.c" /* yacc.c:1646  */
+#line 1740 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 35:
-#line 394 "go-exp.y" /* yacc.c:1646  */
+#line 393 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LSH); }
-#line 1747 "go-exp.c" /* yacc.c:1646  */
+#line 1746 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 36:
-#line 398 "go-exp.y" /* yacc.c:1646  */
+#line 397 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_RSH); }
-#line 1753 "go-exp.c" /* yacc.c:1646  */
+#line 1752 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 37:
-#line 402 "go-exp.y" /* yacc.c:1646  */
+#line 401 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_EQUAL); }
-#line 1759 "go-exp.c" /* yacc.c:1646  */
+#line 1758 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 38:
-#line 406 "go-exp.y" /* yacc.c:1646  */
+#line 405 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_NOTEQUAL); }
-#line 1765 "go-exp.c" /* yacc.c:1646  */
+#line 1764 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 39:
-#line 410 "go-exp.y" /* yacc.c:1646  */
+#line 409 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LEQ); }
-#line 1771 "go-exp.c" /* yacc.c:1646  */
+#line 1770 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 40:
-#line 414 "go-exp.y" /* yacc.c:1646  */
+#line 413 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_GEQ); }
-#line 1777 "go-exp.c" /* yacc.c:1646  */
+#line 1776 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 41:
-#line 418 "go-exp.y" /* yacc.c:1646  */
+#line 417 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LESS); }
-#line 1783 "go-exp.c" /* yacc.c:1646  */
+#line 1782 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 42:
-#line 422 "go-exp.y" /* yacc.c:1646  */
+#line 421 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_GTR); }
-#line 1789 "go-exp.c" /* yacc.c:1646  */
+#line 1788 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 43:
-#line 426 "go-exp.y" /* yacc.c:1646  */
+#line 425 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_AND); }
-#line 1795 "go-exp.c" /* yacc.c:1646  */
+#line 1794 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 44:
-#line 430 "go-exp.y" /* yacc.c:1646  */
+#line 429 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_XOR); }
-#line 1801 "go-exp.c" /* yacc.c:1646  */
+#line 1800 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 45:
-#line 434 "go-exp.y" /* yacc.c:1646  */
+#line 433 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_IOR); }
-#line 1807 "go-exp.c" /* yacc.c:1646  */
+#line 1806 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 46:
-#line 438 "go-exp.y" /* yacc.c:1646  */
+#line 437 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LOGICAL_AND); }
-#line 1813 "go-exp.c" /* yacc.c:1646  */
+#line 1812 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 47:
-#line 442 "go-exp.y" /* yacc.c:1646  */
+#line 441 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_LOGICAL_OR); }
-#line 1819 "go-exp.c" /* yacc.c:1646  */
+#line 1818 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 48:
-#line 446 "go-exp.y" /* yacc.c:1646  */
+#line 445 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, TERNOP_COND); }
-#line 1825 "go-exp.c" /* yacc.c:1646  */
+#line 1824 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 49:
-#line 450 "go-exp.y" /* yacc.c:1646  */
+#line 449 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_ASSIGN); }
-#line 1831 "go-exp.c" /* yacc.c:1646  */
+#line 1830 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 50:
-#line 454 "go-exp.y" /* yacc.c:1646  */
+#line 453 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, BINOP_ASSIGN_MODIFY);
 			  write_exp_elt_opcode (pstate, (yyvsp[-1].opcode));
 			  write_exp_elt_opcode (pstate, BINOP_ASSIGN_MODIFY); }
-#line 1839 "go-exp.c" /* yacc.c:1646  */
+#line 1838 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 51:
-#line 460 "go-exp.y" /* yacc.c:1646  */
+#line 459 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, OP_LONG);
 			  write_exp_elt_type (pstate, (yyvsp[0].typed_val_int).type);
 			  write_exp_elt_longcst (pstate, (LONGEST)((yyvsp[0].typed_val_int).val));
 			  write_exp_elt_opcode (pstate, OP_LONG); }
-#line 1848 "go-exp.c" /* yacc.c:1646  */
+#line 1847 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 52:
-#line 467 "go-exp.y" /* yacc.c:1646  */
+#line 466 "go-exp.y" /* yacc.c:1646  */
     {
 			  struct stoken_vector vec;
 			  vec.len = 1;
 			  vec.tokens = &(yyvsp[0].tsval);
 			  write_exp_string_vector (pstate, (yyvsp[0].tsval).type, &vec);
 			}
-#line 1859 "go-exp.c" /* yacc.c:1646  */
+#line 1858 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 53:
-#line 476 "go-exp.y" /* yacc.c:1646  */
+#line 475 "go-exp.y" /* yacc.c:1646  */
     { YYSTYPE val;
 			  parse_number (pstate, (yyvsp[0].ssym).stoken.ptr,
 					(yyvsp[0].ssym).stoken.length, 0, &val);
@@ -1868,28 +1867,28 @@ yyreduce:
 						 val.typed_val_int.val);
 			  write_exp_elt_opcode (pstate, OP_LONG);
 			}
-#line 1873 "go-exp.c" /* yacc.c:1646  */
+#line 1872 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 54:
-#line 489 "go-exp.y" /* yacc.c:1646  */
+#line 488 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, OP_DOUBLE);
 			  write_exp_elt_type (pstate, (yyvsp[0].typed_val_float).type);
 			  write_exp_elt_dblcst (pstate, (yyvsp[0].typed_val_float).dval);
 			  write_exp_elt_opcode (pstate, OP_DOUBLE); }
-#line 1882 "go-exp.c" /* yacc.c:1646  */
+#line 1881 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 56:
-#line 499 "go-exp.y" /* yacc.c:1646  */
+#line 498 "go-exp.y" /* yacc.c:1646  */
     {
 			  write_dollar_variable (pstate, (yyvsp[0].sval));
 			}
-#line 1890 "go-exp.c" /* yacc.c:1646  */
+#line 1889 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 57:
-#line 505 "go-exp.y" /* yacc.c:1646  */
+#line 504 "go-exp.y" /* yacc.c:1646  */
     {
 			  /* TODO(dje): Go objects in structs.  */
 			  write_exp_elt_opcode (pstate, OP_LONG);
@@ -1902,20 +1901,20 @@ yyreduce:
 						 (LONGEST) TYPE_LENGTH ((yyvsp[-1].tval)));
 			  write_exp_elt_opcode (pstate, OP_LONG);
 			}
-#line 1907 "go-exp.c" /* yacc.c:1646  */
+#line 1906 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 58:
-#line 520 "go-exp.y" /* yacc.c:1646  */
+#line 519 "go-exp.y" /* yacc.c:1646  */
     {
 			  /* TODO(dje): Go objects in structs.  */
 			  write_exp_elt_opcode (pstate, UNOP_SIZEOF);
 			}
-#line 1916 "go-exp.c" /* yacc.c:1646  */
+#line 1915 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 59:
-#line 527 "go-exp.y" /* yacc.c:1646  */
+#line 526 "go-exp.y" /* yacc.c:1646  */
     {
 			  /* We copy the string here, and not in the
 			     lexer, to guarantee that we do not leak a
@@ -1931,11 +1930,11 @@ yyreduce:
 			  vec->ptr = xmalloc ((yyvsp[0].tsval).length + 1);
 			  memcpy (vec->ptr, (yyvsp[0].tsval).ptr, (yyvsp[0].tsval).length + 1);
 			}
-#line 1936 "go-exp.c" /* yacc.c:1646  */
+#line 1935 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 60:
-#line 544 "go-exp.y" /* yacc.c:1646  */
+#line 543 "go-exp.y" /* yacc.c:1646  */
     {
 			  /* Note that we NUL-terminate here, but just
 			     for convenience.  */
@@ -1951,11 +1950,11 @@ yyreduce:
 			  (yyval.svec).tokens[(yyval.svec).len - 1].length = (yyvsp[0].tsval).length;
 			  (yyval.svec).tokens[(yyval.svec).len - 1].ptr = p;
 			}
-#line 1956 "go-exp.c" /* yacc.c:1646  */
+#line 1955 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 61:
-#line 562 "go-exp.y" /* yacc.c:1646  */
+#line 561 "go-exp.y" /* yacc.c:1646  */
     {
 			  int i;
 
@@ -1965,27 +1964,27 @@ yyreduce:
 			    xfree ((yyvsp[0].svec).tokens[i].ptr);
 			  xfree ((yyvsp[0].svec).tokens);
 			}
-#line 1970 "go-exp.c" /* yacc.c:1646  */
+#line 1969 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 62:
-#line 574 "go-exp.y" /* yacc.c:1646  */
+#line 573 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, OP_BOOL);
 			  write_exp_elt_longcst (pstate, (LONGEST) (yyvsp[0].lval));
 			  write_exp_elt_opcode (pstate, OP_BOOL); }
-#line 1978 "go-exp.c" /* yacc.c:1646  */
+#line 1977 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 63:
-#line 580 "go-exp.y" /* yacc.c:1646  */
+#line 579 "go-exp.y" /* yacc.c:1646  */
     { write_exp_elt_opcode (pstate, OP_BOOL);
 			  write_exp_elt_longcst (pstate, (LONGEST) (yyvsp[0].lval));
 			  write_exp_elt_opcode (pstate, OP_BOOL); }
-#line 1986 "go-exp.c" /* yacc.c:1646  */
+#line 1985 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 64:
-#line 586 "go-exp.y" /* yacc.c:1646  */
+#line 585 "go-exp.y" /* yacc.c:1646  */
     { struct symbol *sym = (yyvsp[-1].ssym).sym;
 
 			  if (sym == NULL
@@ -1999,11 +1998,11 @@ yyreduce:
 			  write_exp_elt_sym (pstate, sym);
 			  write_exp_elt_opcode (pstate, OP_VAR_ENTRY_VALUE);
 			}
-#line 2004 "go-exp.c" /* yacc.c:1646  */
+#line 2003 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 65:
-#line 602 "go-exp.y" /* yacc.c:1646  */
+#line 601 "go-exp.y" /* yacc.c:1646  */
     { struct symbol *sym = (yyvsp[0].ssym).sym;
 
 			  if (sym)
@@ -2048,30 +2047,30 @@ yyreduce:
 				       copy_name ((yyvsp[0].ssym).stoken));
 			    }
 			}
-#line 2053 "go-exp.c" /* yacc.c:1646  */
+#line 2052 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 66:
-#line 657 "go-exp.y" /* yacc.c:1646  */
+#line 656 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = lookup_pointer_type ((yyvsp[0].tval)); }
-#line 2059 "go-exp.c" /* yacc.c:1646  */
+#line 2058 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 67:
-#line 659 "go-exp.y" /* yacc.c:1646  */
+#line 658 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = (yyvsp[0].tsym).type; }
-#line 2065 "go-exp.c" /* yacc.c:1646  */
+#line 2064 "go-exp.c" /* yacc.c:1646  */
     break;
 
   case 68:
-#line 666 "go-exp.y" /* yacc.c:1646  */
+#line 665 "go-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = builtin_go_type (parse_gdbarch (pstate))
 			    ->builtin_uint8; }
-#line 2072 "go-exp.c" /* yacc.c:1646  */
+#line 2071 "go-exp.c" /* yacc.c:1646  */
     break;
 
 
-#line 2076 "go-exp.c" /* yacc.c:1646  */
+#line 2075 "go-exp.c" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2299,7 +2298,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 688 "go-exp.y" /* yacc.c:1906  */
+#line 687 "go-exp.y" /* yacc.c:1906  */
 
 
 /* Wrapper on parse_c_float to get the type right for Go.  */
@@ -2604,7 +2603,7 @@ parse_string_or_char (const char *tokptr, const char **outptr,
 
 struct token
 {
-  char *operator;
+  char *oper;
   int token;
   enum exp_opcode opcode;
 };
@@ -2691,7 +2690,7 @@ lex_one_token (struct parser_state *par_state)
   tokstart = lexptr;
   /* See if it is a special token of length 3.  */
   for (i = 0; i < sizeof (tokentab3) / sizeof (tokentab3[0]); i++)
-    if (strncmp (tokstart, tokentab3[i].operator, 3) == 0)
+    if (strncmp (tokstart, tokentab3[i].oper, 3) == 0)
       {
 	lexptr += 3;
 	yylval.opcode = tokentab3[i].opcode;
@@ -2700,7 +2699,7 @@ lex_one_token (struct parser_state *par_state)
 
   /* See if it is a special token of length 2.  */
   for (i = 0; i < sizeof (tokentab2) / sizeof (tokentab2[0]); i++)
-    if (strncmp (tokstart, tokentab2[i].operator, 2) == 0)
+    if (strncmp (tokstart, tokentab2[i].oper, 2) == 0)
       {
 	lexptr += 2;
 	yylval.opcode = tokentab2[i].opcode;
@@ -2931,7 +2930,7 @@ lex_one_token (struct parser_state *par_state)
   /* Catch specific keywords.  */
   copy = copy_name (yylval.sval);
   for (i = 0; i < sizeof (ident_tokens) / sizeof (ident_tokens[0]); i++)
-    if (strcmp (copy, ident_tokens[i].operator) == 0)
+    if (strcmp (copy, ident_tokens[i].oper) == 0)
       {
 	/* It is ok to always set this, even though we don't always
 	   strictly need to.  */
@@ -3071,9 +3070,9 @@ classify_name (struct parser_state *par_state, const struct block *block)
   copy = copy_name (yylval.sval);
 
   /* Try primitive types first so they win over bad/weird debug info.  */
-  type = language_lookup_primitive_type_by_name (parse_language (par_state),
-						 parse_gdbarch (par_state),
-						 copy);
+  type = language_lookup_primitive_type (parse_language (par_state),
+					 parse_gdbarch (par_state),
+					 copy);
   if (type != NULL)
     {
       /* NOTE: We take advantage of the fact that yylval coming in was a
